@@ -30,6 +30,28 @@ NumericVector dgCMatrix_rowSums2(S4 matrix, bool na_rm){
   return wrap(result);
 }
 
+// [[Rcpp::export]]
+NumericVector dgCMatrix_rowSums2_col_select(S4 matrix, bool na_rm, LogicalVector col_selector){
+  IntegerVector dim = matrix.slot("Dim");
+  NumericVector values = matrix.slot("x");
+  IntegerVector row_indices = matrix.slot("i");
+  IntegerVector pointers = matrix.slot("p");
+  std::vector<LDOUBLE> result (dim[0], 0.0);
+  int n_cols = dim[1];
+
+  for(int i = 0; i < n_cols; ++i){
+    if(col_selector[i] == 1){
+      int val_idx = pointers[i];
+      int end_idx = pointers[i + 1];
+      for(;val_idx < end_idx; ++val_idx){
+        result[row_indices[val_idx]] += values[val_idx];
+      }
+    }
+  }
+
+  return wrap(result);
+}
+
 
 
 // [[Rcpp::export]]
