@@ -3,16 +3,18 @@ set.seed(1)
 
 mat <- t(make_matrix_with_all_features(nrow=15, ncol=10))
 rownames(mat) <- letters[seq_len(10)]
+colnames(mat) <- paste0("col_", seq_len(ncol(mat)))
 sp_mat <- as(mat, "dgCMatrix")
 row_subset <- 1:5
 col_subset <- c(7, 9, 2)
 
-skip_test_because_of_name_problems <- TRUE
-
 test_that("rowSums works", {
   expect_equal(rowSums2(sp_mat), MatrixGenerics::rowSums2(mat))
   expect_equal(rowSums2(sp_mat, na.rm=TRUE), MatrixGenerics::rowSums2(mat, na.rm=TRUE))
   expect_equal(rowSums2(sp_mat, rows = row_subset, cols = col_subset), MatrixGenerics::rowSums2(mat, rows = row_subset, cols = col_subset))
+
+  col_selector <- sample(c(TRUE, FALSE), size = ncol(mat), replace = TRUE)
+  expect_equal(rowSums2(sp_mat, rows = row_subset, cols = col_selector), MatrixGenerics::rowSums2(mat, rows = row_subset, cols = col_selector))
 })
 
 
@@ -214,15 +216,11 @@ test_that("rowRanks works", {
 
 
 test_that("rowWeightedMeans works", {
-  # matrixStats has a bug (#175) that rowWeightedMeans returns a vector
-  # without names if w != NULL
-  # As a work around, I set the names of my result to NULL as well
   weights <- rnorm(ncol(sp_mat), mean=4, sd=0.1)
   expect_equal(rowWeightedMeans(sp_mat, w=NULL), MatrixGenerics::rowWeightedMeans(mat, w=NULL))
-  if(skip_test_because_of_name_problems) skip("Naming incompatible")
-  expect_equal(unname(rowWeightedMeans(sp_mat, w=weights)), MatrixGenerics::rowWeightedMeans(mat, w=weights))
+  expect_equal(rowWeightedMeans(sp_mat, w=weights), MatrixGenerics::rowWeightedMeans(mat, w=weights))
   expect_equal(rowWeightedMeans(sp_mat, na.rm=TRUE, w=weights), MatrixGenerics::rowWeightedMeans(mat, na.rm=TRUE, w=weights))
-  expect_equal(unname(rowWeightedMeans(sp_mat, w=weights, rows = row_subset, cols = col_subset)), MatrixGenerics::rowWeightedMeans(mat, w=weights, rows = row_subset, cols = col_subset))
+  expect_equal(rowWeightedMeans(sp_mat, w=weights, rows = row_subset, cols = col_subset), MatrixGenerics::rowWeightedMeans(mat, w=weights, rows = row_subset, cols = col_subset))
 })
 
 
@@ -298,7 +296,7 @@ test_that("rowCollapse works", {
   expect_equal(rowCollapse(sp_mat, idxs = 1), MatrixGenerics::rowCollapse(mat, idxs = 1))
   expect_equal(rowCollapse(sp_mat, idxs = c(1,3)), MatrixGenerics::rowCollapse(mat, idxs = c(1,3)))
   expect_equal(rowCollapse(sp_mat, idxs = 1:5, rows = 3), MatrixGenerics::rowCollapse(mat, idxs = 1:5, rows = 3))
-  expect_equal(rowCollapse(sp_mat, idxs = 1, rows = row_subset), unname(mat[row_subset, 1]))
+  expect_equal(rowCollapse(sp_mat, idxs = 1, rows = row_subset), mat[row_subset, 1])
   expect_equal(rowCollapse(sp_mat, idxs = 1, rows = row_subset), MatrixGenerics::rowCollapse(mat, idxs = 1, rows = row_subset))
 
 })
